/*
 * ################################################################
 *
 * ProActive Parallel Suite(TM): The Java(TM) library for
 *    Parallel, Distributed, Multi-Core Computing for
 *    Enterprise Grids & Clouds
 *
 * Copyright (C) 1997-2012 INRIA/University of
 *                 Nice-Sophia Antipolis/ActiveEon
 * Contact: proactive@ow2.org or contact@activeeon.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; version 3 of
 * the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * If needed, contact us to obtain a release under GPL Version 2 or 3
 * or a different license than the AGPL.
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 */
package org.objectweb.proactive.mavenplugin;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.maven.plugin.MojoExecutionException;


/**
 * Mojo used to create ProActive stubs for the specified classes.
 * 
 * @goal stubs
 * 
 * @author The ProActive Team
 */
public class StubsMojo extends AbstractClassGeneratorMojo {

    private static final String UTILS_CLASSNAME = "org.objectweb.proactive.core.mop.Utils";

    private static final String JAVASSIST_BYTE_CODE_STUB_BUILDER_CLASSNAME = "org.objectweb.proactive.core.mop.JavassistByteCodeStubBuilder";

    /**
     * A list of inclusions when searching for classes to compile.
     * 
     * @parameter
     * @required
     */
    private List<String> includes;

    private Method createMethod;

    private Method convertClassNameToStubClassNameMethod;

    private Method convertStubClassNameToClassNameMethod;

    protected void init() throws MojoExecutionException {
        try {
            Class<?> utilsClass = this.classLoader.loadClass(UTILS_CLASSNAME);
            this.convertClassNameToStubClassNameMethod = utilsClass.getMethod(
                    "convertClassNameToStubClassName", String.class, Class[].class);
            this.convertStubClassNameToClassNameMethod = utilsClass.getMethod(
                    "convertStubClassNameToClassName", String.class);
            Class<?> javassistByteCodeStubBuilderClass = this.classLoader
                    .loadClass(JAVASSIST_BYTE_CODE_STUB_BUILDER_CLASSNAME);
            this.createMethod = javassistByteCodeStubBuilderClass.getMethod("create", String.class,
                    Class[].class);
        } catch (ClassNotFoundException cnfe) {
            throw new MojoExecutionException(
                "ProActive Programming is not a dependency or a transitive dependency of the current module");
        } catch (Exception e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }

    protected List<String> getClassNames() throws MojoExecutionException {
        try {
            List<String> classNames = new ArrayList<String>();

            for (String include : this.includes) {
                classNames.add(this.getStubClassName(include));
            }

            return classNames;
        } catch (Exception e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }

    private String getStubClassName(String className) throws Exception {
        return (String) (this.convertClassNameToStubClassNameMethod.invoke(null, className, null));
    }

    protected byte[] generateClass(String className) throws Exception {
        return (byte[]) (this.createMethod.invoke(null, this.getObjectClassName(className), null));
    }

    private String getObjectClassName(String stubClassName) throws Exception {
        return (String) (this.convertStubClassNameToClassNameMethod.invoke(null, stubClassName));
    }

    protected String getKind() {
        return "ProActive stub";
    }

}
